@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,17 @@
 """DatasetBuilder base class."""
 
 import abc
+import dataclasses
 import functools
 import inspect
 import json
 import os
 import sys
-from typing import Any, ClassVar, Dict, Iterable, List, Optional, Type, Union
+from typing import Any, ClassVar, Dict, Iterable, List, Optional, Tuple, Type, Union
 
 from absl import logging
-import dataclasses
 import six
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import constants
 from tensorflow_datasets.core import dataset_info
@@ -34,6 +34,7 @@ from tensorflow_datasets.core import decode
 from tensorflow_datasets.core import download
 from tensorflow_datasets.core import file_adapters
 from tensorflow_datasets.core import logging as tfds_logging
+from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import registered
 from tensorflow_datasets.core import split_builder as split_builder_lib
 from tensorflow_datasets.core import splits as splits_lib
@@ -41,6 +42,8 @@ from tensorflow_datasets.core import tf_compat
 from tensorflow_datasets.core import tfrecords_reader
 from tensorflow_datasets.core import units
 from tensorflow_datasets.core import utils
+from tensorflow_datasets.core.proto import dataset_info_pb2
+from tensorflow_datasets.core.utils import file_utils
 from tensorflow_datasets.core.utils import gcs_utils
 from tensorflow_datasets.core.utils import read_config as read_config_lib
 from tensorflow_datasets.core.utils import type_utils
@@ -49,6 +52,7 @@ import termcolor
 
 ReadOnlyPath = type_utils.ReadOnlyPath
 ReadWritePath = type_utils.ReadWritePath
+Tree = type_utils.Tree
 TreeDict = type_utils.TreeDict
 VersionOrStr = Union[utils.Version, str]
 
@@ -78,9 +82,23 @@ class BuilderConfig:
   name: str
   version: Optional[VersionOrStr] = None
   release_notes: Optional[Dict[str, str]] = None
-  supported_versions: List[str] = dataclasses.field(default_factory=list)
+  supported_versions: List[VersionOrStr] = dataclasses.field(
+      default_factory=list)
   description: Optional[str] = None
 
+  @classmethod
+  def from_dataset_info(
+      cls,
+      info_proto: dataset_info_pb2.DatasetInfo) -> Optional["BuilderConfig"]:
+    if not info_proto.config_name:
+      return None
+    return BuilderConfig(
+        name=info_proto.config_name,
+        description=info_proto.config_description,
+        version=info_proto.version,
+        release_notes=info_proto.release_notes or {},
+    )
+
 
 class DatasetBuilder(registered.RegisteredDataset):
   """Abstract base class for all datasets.
@@ -350,6 +368,11 @@ class DatasetBuilder(registered.RegisteredDataset):
     if data_exists and download_config.download_mode == REUSE_DATASET_IF_EXISTS:
       logging.info("Reusing dataset %s (%s)", self.name, self._data_dir)
       return
+    elif data_exists and download_config.download_mode == REUSE_CACHE_IF_EXISTS:
+      logging.info("Deleting pre-existing dataset %s (%s)", self.name,
+                   self._data_dir)
+      utils.as_path(self._data_dir).rmtree()  # Delete pre-existing data.
+      data_exists = tf.io.gfile.exists(self._data_dir)
 
     if self.version.tfds_version_to_prepare:
       available_to_prepare = ", ".join(
@@ -453,11 +476,11 @@ class DatasetBuilder(registered.RegisteredDataset):
   @tfds_logging.as_dataset()
   def as_dataset(
       self,
-      split: Optional[Union[str, tfrecords_reader.ReadInstruction]] = None,
+      split: Optional[Tree[splits_lib.SplitArg]] = None,
       *,
       batch_size: Optional[int] = None,
       shuffle_files: bool = False,
-      decoders: Optional[TreeDict[decode.Decoder]] = None,
+      decoders: Optional[TreeDict[decode.partial_decode.DecoderArg]] = None,
       read_config: Optional[read_config_lib.ReadConfig] = None,
       as_supervised: bool = False,
   ):
@@ -560,16 +583,16 @@ class DatasetBuilder(registered.RegisteredDataset):
         read_config=read_config,
         as_supervised=as_supervised,
     )
-    datasets = utils.map_nested(build_single_dataset, split, map_tuple=True)
-    return datasets
+    all_ds = tf.nest.map_structure(build_single_dataset, split)
+    return all_ds
 
   def _build_single_dataset(
       self,
       split,
       shuffle_files,
       batch_size,
-      decoders,
-      read_config,
+      decoders: Optional[TreeDict[decode.partial_decode.DecoderArg]],
+      read_config: read_config_lib.ReadConfig,
       as_supervised,
   ):
     """as_dataset for a single split."""
@@ -596,10 +619,25 @@ class DatasetBuilder(registered.RegisteredDataset):
     if as_supervised:
       if not self.info.supervised_keys:
         raise ValueError(
-            "as_supervised=True but %s does not support a supervised "
-            "(input, label) structure." % self.name)
-      input_f, target_f = self.info.supervised_keys
-      ds = ds.map(lambda fs: (fs[input_f], fs[target_f]))
+            f"as_supervised=True but {self.name} does not support a supervised "
+            "structure.")
+
+      def lookup_nest(features: Dict[str, Any]) -> Tuple[Any, ...]:
+        """Converts `features` to the structure described by `supervised_keys`.
+
+        Note that there is currently no way to access features in nested
+        feature dictionaries.
+
+        Args:
+          features: dictionary of features
+
+        Returns:
+          A tuple with elements structured according to `supervised_keys`
+        """
+        return tf.nest.map_structure(lambda key: features[key],
+                                     self.info.supervised_keys)
+
+      ds = ds.map(lookup_nest)
 
     # Add prefetch by default
     if not read_config.skip_prefetch:
@@ -609,17 +647,16 @@ class DatasetBuilder(registered.RegisteredDataset):
     # non-deterministic
     # This code should probably be moved inside tfreader, such as
     # all the tf.data.Options are centralized in a single place.
-    if (shuffle_files and
-        read_config.options.experimental_deterministic is None and
-        read_config.shuffle_seed is None):
+    if (shuffle_files and read_config.shuffle_seed is None and
+        tf_compat.get_option_deterministic(read_config.options) is None):
       options = tf.data.Options()
-      options.experimental_deterministic = False
+      tf_compat.set_option_deterministic(options, False)
       ds = ds.with_options(options)
     # If shuffle is False, keep the default value (deterministic), which
     # allow the user to overwritte it.
 
     if wants_full_dataset:
-      return tf.data.experimental.get_single_element(ds)
+      return tf_compat.get_single_element(ds)
     return ds
 
   def _should_cache_ds(self, split, shuffle_files, read_config):
@@ -687,9 +724,9 @@ class DatasetBuilder(registered.RegisteredDataset):
     builder_dir = self._relative_data_dir(with_version=False)
     version_dir = self._relative_data_dir(with_version=True)
 
-    default_data_dir = constants.get_default_data_dir(
+    default_data_dir = file_utils.get_default_data_dir(
         given_data_dir=given_data_dir)
-    all_data_dirs = constants.list_data_dirs(given_data_dir=given_data_dir)
+    all_data_dirs = file_utils.list_data_dirs(given_data_dir=given_data_dir)
 
     all_versions = set()
     requested_version_dirs = {}
@@ -769,11 +806,13 @@ class DatasetBuilder(registered.RegisteredDataset):
     raise NotImplementedError
 
   @abc.abstractmethod
-  def _as_dataset(self,
-                  split,
-                  decoders=None,
-                  read_config=None,
-                  shuffle_files=False):
+  def _as_dataset(
+      self,
+      split: splits_lib.Split,
+      decoders: Optional[TreeDict[decode.partial_decode.DecoderArg]] = None,
+      read_config: Optional[read_config_lib.ReadConfig] = None,
+      shuffle_files: bool = False,
+  ) -> tf.data.Dataset:
     """Constructs a `tf.data.Dataset`.
 
     Internal implementation to overwrite when inheriting from DatasetBuilder.
@@ -883,10 +922,12 @@ class FileReaderBuilder(DatasetBuilder):
 
   """
 
-  def __init__(self,
-               *,
-               file_format: Union[None, str, file_adapters.FileFormat] = None,
-               **kwargs: Any):
+  def __init__(
+      self,
+      *,
+      file_format: Union[None, str, file_adapters.FileFormat] = None,
+      **kwargs: Any,
+  ):
     """Initializes an instance of FileReaderBuilder.
 
     Callers must pass arguments as keyword arguments.
@@ -906,12 +947,13 @@ class FileReaderBuilder(DatasetBuilder):
 
   def _as_dataset(
       self,
-      split=splits_lib.Split.TRAIN,
-      decoders=None,
-      read_config=None,
-      shuffle_files=False,
+      split: splits_lib.Split,
+      decoders: Optional[TreeDict[decode.partial_decode.DecoderArg]],
+      read_config: read_config_lib.ReadConfig,
+      shuffle_files: bool,
   ) -> tf.data.Dataset:
     # Partial decoding
+    # TODO(epot): Should be moved inside `features.decode_example`
     if isinstance(decoders, decode.PartialDecoding):
       features = decoders.extract_features(self.info.features)
       example_specs = features.get_serialized_info()
@@ -1105,6 +1147,8 @@ class GeneratorBasedBuilder(FileReaderBuilder):
       # forward it to `self._split_generators`
       # We add this magic because the pipeline kwargs is only used by c4 and
       # we do not want to make the API more verbose for a single advanced case.
+      # See also the documentation at the end here:
+      # https://www.tensorflow.org/datasets/api_docs/python/tfds/core/GeneratorBasedBuilder?version=nightly#_generate_examples
       signature = inspect.signature(self._split_generators)
       if "pipeline" in signature.parameters.keys():
         optional_pipeline_kwargs = dict(pipeline=split_builder.beam_pipeline)
@@ -1132,19 +1176,26 @@ class GeneratorBasedBuilder(FileReaderBuilder):
       path_suffix = file_adapters.ADAPTER_FOR_FORMAT[
           self.info.file_format].FILE_SUFFIX
 
-      split_info_futures = [
-          split_builder.submit_split_generation(  # pylint: disable=g-complex-comprehension
-              split_name=split_name,
-              generator=generator,
-              path=self.data_path / f"{self.name}-{split_name}.{path_suffix}",
-              disable_shuffling=self.info.disable_shuffling,
-          ) for split_name, generator in utils.tqdm(
-              split_generators.items(),
-              desc="Generating splits...",
-              unit=" splits",
-              leave=False,
-          )
-      ]
+      split_info_futures = []
+      for split_name, generator in utils.tqdm(
+          split_generators.items(),
+          desc="Generating splits...",
+          unit=" splits",
+          leave=False,
+      ):
+        filename_template = naming.ShardedFileTemplate(
+            split=split_name,
+            dataset_name=self.name,
+            data_dir=self.data_path,
+            filetype_suffix=path_suffix)
+        future = split_builder.submit_split_generation(
+            split_name=split_name,
+            generator=generator,
+            filename_template=filename_template,
+            disable_shuffling=self.info.disable_shuffling,
+        )
+        split_info_futures.append(future)
+
     # Finalize the splits (after apache beam completed, if it was used)
     split_infos = [future.result() for future in split_info_futures]
 
@@ -1197,7 +1248,7 @@ def _save_default_config_name(
 
 def load_default_config_name(common_dir: ReadOnlyPath,) -> Optional[str]:
   """Load `builder_cls` metadata (common to all builder configs)."""
-  config_path = common_dir / ".config/metadata.json"
+  config_path = utils.as_path(common_dir) / ".config/metadata.json"
   if not config_path.exists():
     return None
   data = json.loads(config_path.read_text())
