@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,13 +24,13 @@ import multiprocessing
 import os
 import tarfile
 import typing
-from typing import Iterator, Tuple
+from typing import Iterator, Tuple, Union
 import uuid
 import zipfile
 
 from absl import logging
 import promise
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import constants
 from tensorflow_datasets.core import utils
@@ -228,7 +228,7 @@ _EXTRACT_METHODS = {
 
 
 def iter_archive(
-    path: utils.PathLike,
+    path: Union[utils.PathLike, typing.BinaryIO],
     method: resource_lib.ExtractMethod,
 ) -> Iterator[Tuple[str, typing.BinaryIO]]:
   """Iterate over an archive.
