@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,14 +15,17 @@
 
 """Sequence feature."""
 
+from typing import Optional
+
 import numpy as np
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import utils
 from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.core.features import features_dict
 from tensorflow_datasets.core.features import tensor_feature
 from tensorflow_datasets.core.features import top_level_feature
+from tensorflow_datasets.core.utils import py_utils
 from tensorflow_datasets.core.utils import type_utils
 
 Json = type_utils.Json
@@ -81,19 +84,21 @@ class Sequence(top_level_feature.TopLevelFeature):
 
   """
 
-  def __init__(self, feature, length=None, **kwargs):
+  def __init__(
+      self,
+      feature: feature_lib.FeatureConnectorArg,
+      length: Optional[int] = None,
+  ):
     """Construct a sequence dict.
 
     Args:
-      feature: `dict`, the features to wrap
+      feature: The features to wrap (any feature supported)
       length: `int`, length of the sequence if static and known in advance
-      **kwargs: `dict`, constructor kwargs of `tfds.features.FeaturesDict`
     """
     # Convert {} => FeaturesDict, tf.int32 => Tensor(shape=(), dtype=tf.int32)
     self._feature = features_dict.to_feature(feature)
     self._length = length
-    assert not kwargs, 'Json export/import should be updated'
-    super(Sequence, self).__init__(**kwargs)
+    super(Sequence, self).__init__()
 
   @property
   def feature(self):
@@ -107,12 +112,14 @@ class Sequence(top_level_feature.TopLevelFeature):
     tensor_info.sequence_rank += 1
     return tensor_info
 
+  @py_utils.memoize()
   def get_tensor_info(self):
     """See base class for details."""
     # Add the additional length dimension to every shape
     tensor_info = self._feature.get_tensor_info()
     return tf.nest.map_structure(self._add_length_dim, tensor_info)
 
+  @py_utils.memoize()
   def get_serialized_info(self):
     """See base class for details."""
     # Add the additional length dimension to every serialized features
@@ -162,10 +169,10 @@ class Sequence(top_level_feature.TopLevelFeature):
 
   def __getitem__(self, key):
     """Convenience method to access the underlying features."""
-    return self._feature[key]
+    return self._feature[key]  # pytype: disable=unsupported-operands
 
   def __contains__(self, key: str) -> bool:
-    return key in self._feature
+    return key in self._feature  # pytype: disable=unsupported-operands
 
   def __getattr__(self, key):
     """Allow to access the underlying attributes directly."""
@@ -204,16 +211,16 @@ class Sequence(top_level_feature.TopLevelFeature):
     }
 
 
-def build_empty_np(serialized_info):
+def build_empty_np(serialized_info: feature_lib.TensorInfo):
   """Build empty sequence with the shape of serialized_info."""
   return np.empty(
       shape=tuple(s if s else 0 for s in serialized_info.shape),
-      dtype=serialized_info.dtype.as_numpy_dtype,
+      dtype=serialized_info.numpy_dtype,
   )
 
 
 def stack_nested(sequence_elements):
-  """Recursivelly stack the tensors from the same dict field."""
+  """Recursively stack the tensors from the same dict field."""
   if isinstance(sequence_elements[0], dict):
     return {
         # Stack along the first dimension
@@ -223,7 +230,7 @@ def stack_nested(sequence_elements):
   # Note: As each field can be a nested ragged list, we don't check here
   # that all elements from the list have matching dtype/shape.
   # Checking is done in `example_serializer` when elements
-  # are converted to numpy array and stacked togethers.
+  # are converted to numpy array and stacked together.
   return list(sequence_elements)
 
 
