@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -101,10 +101,12 @@ def builder_cls(name: str) -> Type[dataset_builder.DatasetBuilder]:
     _reraise_with_list_builders(e, name=ds_name)  # pytype: disable=bad-return-type
 
 
-def builder(name: str,
-            *,
-            try_gcs: bool = False,
-            **builder_kwargs: Any) -> dataset_builder.DatasetBuilder:
+def builder(
+    name: str,
+    *,
+    try_gcs: bool = False,
+    **builder_kwargs: Any,
+) -> dataset_builder.DatasetBuilder:
   """Fetches a `tfds.core.DatasetBuilder` by string name.
 
   Args:
@@ -193,13 +195,13 @@ def _try_load_from_files_first(
 def load(
     name: str,
     *,
-    split: Optional[Tree[splits_lib.Split]] = None,
+    split: Optional[Tree[splits_lib.SplitArg]] = None,
     data_dir: Optional[str] = None,
     batch_size: Optional[int] = None,
     shuffle_files: bool = False,
     download: bool = True,
     as_supervised: bool = False,
-    decoders: Optional[TreeDict[decode.Decoder]] = None,
+    decoders: Optional[TreeDict[decode.partial_decode.DecoderArg]] = None,
     read_config: Optional[read_config_lib.ReadConfig] = None,
     with_info: bool = False,
     builder_kwargs: Optional[Dict[str, Any]] = None,
@@ -248,13 +250,18 @@ def load(
 
   Args:
     name: `str`, the registered name of the `DatasetBuilder` (the snake case
-      version of the class name). This can be either `'dataset_name'` or
-      `'dataset_name/config_name'` for datasets with `BuilderConfig`s. As a
-      convenience, this string may contain comma-separated keyword arguments for
-      the builder. For example `'foo_bar/a=True,b=3'` would use the `FooBar`
-      dataset passing the keyword arguments `a=True` and `b=3` (for builders
-      with configs, it would be `'foo_bar/zoo/a=True,b=3'` to use the `'zoo'`
-      config and pass to the builder keyword arguments `a=True` and `b=3`).
+      version of the class name). The config and version can also be specified
+      in the name as follows: `'dataset_name[/config_name][:version]'`. For
+        example, `'movielens/25m-ratings'` (for the latest version of
+      `'25m-ratings'`), `'movielens:0.1.0'` (for the default config and
+      version 0.1.0), or`'movielens/25m-ratings:0.1.0'`. Note that only the
+        latest version can be generated, but old versions can be read if they
+        are present on disk. For convenience, the `name` parameter can contain
+        comma-separated keyword arguments for the builder. For example,
+        `'foo_bar/a=True,b=3'` would use the `FooBar` dataset passing the
+        keyword arguments `a=True` and `b=3` (for builders with configs, it
+        would be `'foo_bar/zoo/a=True,b=3'` to use the `'zoo'` config and pass
+        to the builder keyword arguments `a=True` and `b=3`).
     split: Which split of the data to load (e.g. `'train'`, `'test'`,
       `['train', 'test']`, `'train[80%:]'`,...). See our
       [split API guide](https://www.tensorflow.org/datasets/splits). If `None`,
