@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@ from unittest import mock
 
 import dill
 import pytest
+import tensorflow as tf
 
 from tensorflow_datasets import testing
 from tensorflow_datasets.core import constants
@@ -30,6 +31,9 @@ from tensorflow_datasets.core import dataset_utils
 from tensorflow_datasets.core import load
 from tensorflow_datasets.core import read_only_builder
 from tensorflow_datasets.core import registered
+from tensorflow_datasets.core.features import features_dict
+from tensorflow_datasets.core.proto import dataset_info_pb2
+from tensorflow_datasets.core.utils import file_utils
 
 
 class DummyNoConfMnist(testing.DummyDataset):
@@ -43,11 +47,13 @@ class DummyConfigMnist(testing.DummyDataset):
       dataset_builder.BuilderConfig(
           name='dummy_config',
           version='0.1.0',
+          release_notes={'0.1.0': 'Release notes 0.1.0'},
           description='testing config',
       ),
       dataset_builder.BuilderConfig(
           name='dummy_config2',
           version='0.1.0',
+          release_notes={'0.1.0': 'Release notes 0.1.0'},
           description='testing config',
       ),
   ]
@@ -142,7 +148,7 @@ def test_builder_code_not_found(code_builder: dataset_builder.DatasetBuilder):
 
 
 # Test both with and without config
-def test_read_only_builder(code_builder: dataset_builder.DatasetBuilder):
+def test_builder_from_directory(code_builder: dataset_builder.DatasetBuilder):
   """Builder can be created from the files only."""
 
   # Reconstruct the dataset
@@ -153,6 +159,7 @@ def test_read_only_builder(code_builder: dataset_builder.DatasetBuilder):
   assert builder.info.full_name == code_builder.info.full_name
   assert repr(builder.info) == repr(code_builder.info)
   assert builder.VERSION == code_builder.info.version
+  assert builder.RELEASE_NOTES == code_builder.info.release_notes
   assert builder.__module__ == type(code_builder).__module__
   assert read_only_builder.ReadOnlyBuilder.VERSION is None
 
@@ -178,12 +185,56 @@ def test_read_only_builder(code_builder: dataset_builder.DatasetBuilder):
   assert builder.version == builder2.version
 
 
-def test_not_exists(tmp_path: pathlib.Path):
-  with pytest.raises(
-      FileNotFoundError, match='Could not load `ReadOnlyBuilder`'):
+def test_builder_from_metadata(code_builder: dataset_builder.DatasetBuilder):
+  features = features_dict.FeaturesDict({
+      'a': tf.float32,
+      'b': tf.string,
+  })
+  info_proto = dataset_info_pb2.DatasetInfo(
+      name='abcd',
+      description='efgh',
+      config_name='en',
+      config_description='something',
+      version='9.9.9',
+      release_notes={'9.9.9': 'release description'},
+      citation='some citation',
+      features=features.to_proto())
+  builder = read_only_builder.builder_from_metadata(
+      code_builder.data_dir, info_proto=info_proto)
+  assert builder.name == info_proto.name
+  assert builder.info.description == info_proto.description
+  assert builder.info.citation == info_proto.citation
+  assert builder.info.version == info_proto.version
+  assert builder.builder_config
+  assert builder.builder_config.name == info_proto.config_name
+  assert builder.builder_config.version == info_proto.version
+  assert builder.builder_config.description == info_proto.config_description
+  assert builder.builder_config.release_notes == info_proto.release_notes
+  assert str(builder.info.features) == str(features)
+
+
+def test_builder_from_directory_dir_not_exists(tmp_path: pathlib.Path):
+  with pytest.raises(FileNotFoundError, match='Could not load dataset info'):
     read_only_builder.builder_from_directory(tmp_path)
 
 
+def test_builder_from_files_multi_dir(
+    code_builder: dataset_builder.DatasetBuilder,
+    tmp_path: pathlib.Path,
+):
+  some_dir = tmp_path / 'other'
+  some_dir.mkdir()
+
+  builder = read_only_builder.builder_from_files(
+      code_builder.name,
+      data_dir=[
+          code_builder._data_dir_root, some_dir, '/tmp/non-existing-dir/'
+      ],
+  )
+  assert builder.name == code_builder.name
+  assert builder.data_dir == code_builder.data_dir
+
+
 def test_not_registered():
   """Ensure the ReadOnlyBuilder is not registered."""
   assert read_only_builder.ReadOnlyBuilder.name not in load.list_builders()
@@ -201,7 +252,7 @@ def test_find_builder_dir_with_multiple_data_dir(mock_fs: testing.MockFs):
   assert read_only_builder._find_builder_dir('ds0') is None
 
   with mock.patch.object(
-      constants,
+      file_utils,
       'list_data_dirs',
       return_value=[constants.DATA_DIR, 'path/to'],
   ):
