@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,16 +17,16 @@
 
 import functools
 import itertools
-import os
 from unittest import mock
 
 import six
 
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 import tensorflow_datasets as tfds
 from tensorflow_datasets import testing
 from tensorflow_datasets.core import example_parser
+from tensorflow_datasets.core import naming
 from tensorflow_datasets.core import splits
 from tensorflow_datasets.core import tfrecords_reader
 from tensorflow_datasets.core import tfrecords_writer
@@ -63,240 +63,6 @@ def _write_tfrecord_from_shard_spec(shard_spec, get):
   tfrecords_writer._write_examples(shard_spec.path, itertools.chain(*iterators))
 
 
-class GetDatasetFilesTest(testing.TestCase):
-
-  NAME2SHARD_LENGTHS = {
-      'train': [3, 2, 3, 2, 3],  # 13 examples.
-  }
-
-  PATH_PATTERN = 'mnist-train.tfrecord-0000%d-of-00005'
-
-  def _get_files(self, instruction):
-    file_instructions = tfrecords_reader._make_file_instructions_from_absolutes(
-        name='mnist',
-        name2shard_lengths=self.NAME2SHARD_LENGTHS,
-        absolute_instructions=[instruction],
-    )
-    return file_instructions
-
-  def test_no_skip_no_take(self):
-    instruction = tfrecords_reader._AbsoluteInstruction('train', None, None)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % i, skip=0, take=-1, num_examples=n)
-        for i, n in enumerate([3, 2, 3, 2, 3])
-    ])
-
-  def test_skip(self):
-    # One file is not taken, one file is partially taken.
-    instruction = tfrecords_reader._AbsoluteInstruction('train', 4, None)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 1, skip=1, take=-1, num_examples=1),
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 2, skip=0, take=-1, num_examples=3),
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 3, skip=0, take=-1, num_examples=2),
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 4, skip=0, take=-1, num_examples=3),
-    ])
-
-  def test_take(self):
-    # Two files are not taken, one file is partially taken.
-    instruction = tfrecords_reader._AbsoluteInstruction('train', None, 6)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 0, skip=0, take=-1, num_examples=3),
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 1, skip=0, take=-1, num_examples=2),
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 2, skip=0, take=1, num_examples=1),
-    ])
-
-  def test_skip_take1(self):
-    # A single shard with both skip and take.
-    instruction = tfrecords_reader._AbsoluteInstruction('train', 1, 2)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 0, skip=1, take=1, num_examples=1),
-    ])
-
-  def test_skip_take2(self):
-    # 2 elements in across two shards are taken in middle.
-    instruction = tfrecords_reader._AbsoluteInstruction('train', 7, 9)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 2, skip=2, take=-1, num_examples=1),
-        shard_utils.FileInstruction(
-            filename=self.PATH_PATTERN % 3, skip=0, take=1, num_examples=1),
-    ])
-
-  def test_touching_boundaries(self):
-    # Nothing to read.
-    instruction = tfrecords_reader._AbsoluteInstruction('train', 0, 0)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [])
-
-    instruction = tfrecords_reader._AbsoluteInstruction('train', None, 0)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [])
-
-    instruction = tfrecords_reader._AbsoluteInstruction('train', 3, 3)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [])
-
-    instruction = tfrecords_reader._AbsoluteInstruction('train', 13, None)
-    files = self._get_files(instruction)
-    self.assertEqual(files, [])
-
-  def test_missing_shard_lengths(self):
-    with self.assertRaisesWithPredicateMatch(ValueError, 'Shard empty.'):
-      split_info = [
-          splits.SplitInfo(name='train', shard_lengths=[], num_bytes=0),
-      ]
-      tfrecords_reader.make_file_instructions('mnist', split_info, 'train')
-
-
-class ReadInstructionTest(testing.TestCase):
-
-  def setUp(self):
-    super(ReadInstructionTest, self).setUp()
-    self.splits = {'train': 200, 'test': 101, 'validation': 30, 'dev-train': 10}
-
-  def check_from_ri(self, ri, expected):
-    res = ri.to_absolute(self.splits)
-    expected_result = []
-    for split_name, from_, to_ in expected:
-      expected_result.append(
-          tfrecords_reader._AbsoluteInstruction(split_name, from_, to_))
-    self.assertEqual(res, expected_result)
-    return ri
-
-  def check_from_spec(self, spec, expected):
-    ri = tfrecords_reader.ReadInstruction.from_spec(spec)
-    return self.check_from_ri(ri, expected)
-
-  def assertRaises(self, spec, msg, exc_cls=AssertionError):
-    with self.assertRaisesWithPredicateMatch(exc_cls, msg):
-      ri = tfrecords_reader.ReadInstruction.from_spec(spec)
-      ri.to_absolute(self.splits)
-
-  def test_valid(self):
-    # Simple split:
-    ri = self.check_from_spec('train', [('train', None, None)])
-    self.assertEqual(
-        str(ri),
-        ('ReadInstruction(['
-         "_RelativeInstruction(splitname='train', from_=None, to=None, "
-         "unit='abs', rounding='closest')])"))
-    self.check_from_spec('test', [('test', None, None)])
-    # Addition of splits:
-    self.check_from_spec('train+test', [
-        ('train', None, None),
-        ('test', None, None),
-    ])
-    # Absolute slicing:
-    self.check_from_spec('train[0:0]', [('train', None, 0)])
-    self.check_from_spec('train[:10]', [('train', None, 10)])
-    self.check_from_spec('train[0:10]', [('train', None, 10)])
-    self.check_from_spec('train[-10:]', [('train', 190, None)])
-    self.check_from_spec('train[-100:-50]', [('train', 100, 150)])
-    self.check_from_spec('train[-10:200]', [('train', 190, None)])
-    self.check_from_spec('train[10:-10]', [('train', 10, 190)])
-    self.check_from_spec('train[42:99]', [('train', 42, 99)])
-    # Percent slicing, closest rounding:
-    self.check_from_spec('train[:10%]', [('train', None, 20)])
-    self.check_from_spec('train[90%:]', [('train', 180, None)])
-    self.check_from_spec('train[-1%:]', [('train', 198, None)])
-    ri = self.check_from_spec('test[:99%]', [('test', None, 100)])
-    self.assertEqual(
-        str(ri),
-        ("ReadInstruction([_RelativeInstruction(splitname='test', from_=None,"
-         " to=99, unit='%', rounding='closest')])"))
-    # No overlap:
-    self.check_from_spec('test[100%:]', [('test', 101, None)])
-    # Percent slicing, pct1_dropremainder rounding:
-    ri = tfrecords_reader.ReadInstruction(
-        'train', to=20, unit='%', rounding='pct1_dropremainder')
-    self.check_from_ri(ri, [('train', None, 40)])
-    # test split has 101 examples.
-    ri = tfrecords_reader.ReadInstruction(
-        'test', to=100, unit='%', rounding='pct1_dropremainder')
-    self.check_from_ri(ri, [('test', None, 100)])
-    # No overlap using 'pct1_dropremainder' rounding:
-    ri1 = tfrecords_reader.ReadInstruction(
-        'test', to=99, unit='%', rounding='pct1_dropremainder')
-    ri2 = tfrecords_reader.ReadInstruction(
-        'test', from_=100, unit='%', rounding='pct1_dropremainder')
-    self.check_from_ri(ri1, [('test', None, 99)])
-    self.check_from_ri(ri2, [('test', 100, None)])
-    # Empty:
-    # Slices resulting in empty datasets are valid with 'closest' rounding:
-    self.check_from_spec('validation[:1%]', [('validation', None, 0)])
-
-    # Supports splits with '-' in name.
-    ri = self.check_from_spec('dev-train', [('dev-train', None, None)])
-
-  def test_add(self):
-    ri1 = tfrecords_reader.ReadInstruction.from_spec('train[10:20]')
-    ri2 = tfrecords_reader.ReadInstruction.from_spec('test[10:20]')
-    ri3 = tfrecords_reader.ReadInstruction.from_spec('train[1:5]')
-    ri = ri1 + ri2 + ri3
-    self.assertEqual(
-        str(ri),
-        ('ReadInstruction(['
-         "_RelativeInstruction(splitname='train', from_=10, to=20, unit='abs',"
-         " rounding='closest'), "
-         "_RelativeInstruction(splitname='test', from_=10, to=20, unit='abs',"
-         " rounding='closest'), "
-         "_RelativeInstruction(splitname='train', from_=1, to=5, unit='abs',"
-         " rounding='closest')])"))
-
-  def test_add_invalid(self):
-    # Mixed rounding:
-    ri1 = tfrecords_reader.ReadInstruction(
-        'test', unit='%', to=10, rounding='pct1_dropremainder')
-    ri2 = tfrecords_reader.ReadInstruction(
-        'test', unit='%', from_=90, rounding='closest')
-    with self.assertRaisesWithPredicateMatch(AssertionError,
-                                             'different rounding'):
-      unused_ = ri1 + ri2
-
-  def test_invalid_rounding(self):
-    with self.assertRaisesWithPredicateMatch(ValueError, 'rounding'):
-      tfrecords_reader.ReadInstruction('test', unit='%', rounding='unexisting')
-
-  def test_invalid_unit(self):
-    with self.assertRaisesWithPredicateMatch(ValueError, 'unit'):
-      tfrecords_reader.ReadInstruction('test', unit='kg', rounding='closest')
-
-  def test_invalid_spec(self):
-    # Invalid format:
-    self.assertRaises('validation[:250%:2]',
-                      'Unrecognized instruction format: validation[:250%:2]')
-    # Unexisting split:
-    self.assertRaises(
-        'imaginary', 'Unknown split "imaginary"', exc_cls=ValueError)
-    # Invalid boundaries abs:
-    self.assertRaises('validation[:31]', 'incompatible with 30 examples')
-    # Invalid boundaries %:
-    self.assertRaises('validation[:250%]',
-                      'Percent slice boundaries must be > -100 and < 100')
-    self.assertRaises('validation[-101%:]',
-                      'Percent slice boundaries must be > -100 and < 100')
-    # pct1_dropremainder with < 100 examples
-    with self.assertRaisesWithPredicateMatch(
-        AssertionError, 'with less than 100 elements is forbidden'):
-      ri = tfrecords_reader.ReadInstruction(
-          'validation', to=99, unit='%', rounding='pct1_dropremainder')
-      ri.to_absolute(self.splits)
-
-
 class ReaderTest(testing.TestCase):
 
   def setUp(self):
@@ -311,12 +77,19 @@ class ReaderTest(testing.TestCase):
       )
 
   def _write_tfrecord(self, split_name, shards_number, records):
-    path = os.path.join(self.tmp_dir, 'mnist-%s.tfrecord' % split_name)
+    filename_template = naming.ShardedFileTemplate(
+        dataset_name='mnist',
+        split=split_name,
+        filetype_suffix='tfrecord',
+        data_dir=self.tmp_dir)
     num_examples = len(records)
     with mock.patch.object(
         tfrecords_writer, '_get_number_shards', return_value=shards_number):
-      shard_specs = tfrecords_writer._get_shard_specs(num_examples, 0,
-                                                      [num_examples], path)
+      shard_specs = tfrecords_writer._get_shard_specs(
+          num_examples=num_examples,
+          total_size=0,
+          bucket_lengths=[num_examples],
+          filename_template=filename_template)
     serialized_records = [(key, six.b(rec)) for key, rec in enumerate(records)]
     for shard_spec in shard_specs:
       _write_tfrecord_from_shard_spec(shard_spec,
@@ -329,7 +102,7 @@ class ReaderTest(testing.TestCase):
 
   def test_nodata_instruction(self):
     # Given instruction corresponds to no data.
-    with self.assertRaisesWithPredicateMatch(AssertionError,
+    with self.assertRaisesWithPredicateMatch(ValueError,
                                              'corresponds to no data!'):
       train_info = splits.SplitInfo(
           name='train',
@@ -440,7 +213,7 @@ class ReaderTest(testing.TestCase):
   def test_4fold(self):
     train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
     instructions = [
-        tfrecords_reader.ReadInstruction('train', from_=k, to=k + 25, unit='%')
+        splits.ReadInstruction('train', from_=k, to=k + 25, unit='%')
         for k in range(0, 100, 25)
     ]
     tests = self.reader.read(
@@ -448,11 +221,9 @@ class ReaderTest(testing.TestCase):
         instructions=instructions,
         split_infos=[train_info],
     )
-    instructions = [
-        (tfrecords_reader.ReadInstruction('train', to=k, unit='%') +
-         tfrecords_reader.ReadInstruction('train', from_=k + 25, unit='%'))
-        for k in range(0, 100, 25)
-    ]
+    instructions = [(splits.ReadInstruction('train', to=k, unit='%') +
+                     splits.ReadInstruction('train', from_=k + 25, unit='%'))
+                    for k in range(0, 100, 25)]
     trains = self.reader.read(
         name='mnist',
         instructions=instructions,
@@ -580,6 +351,63 @@ class ReaderTest(testing.TestCase):
       expected_warning = _SHUFFLE_FILES_ERROR_MESSAGE + '\n' + _CYCLE_LENGTH_ERROR_MESSAGE
       self.assertIn(expected_warning, reported_warnings)
 
+  @mock.patch(
+      'tensorflow.data.experimental.assert_cardinality',
+      wraps=tf.data.experimental.assert_cardinality)
+  def test_assert_cardinality_is_on_by_default(self, assert_cardinality):
+    train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
+    self.reader.read(
+        name='mnist', instructions='train', split_infos=[train_info])
+    assert_cardinality.assert_called_with(12)
+
+  @mock.patch('tensorflow.data.experimental.assert_cardinality')
+  def test_assert_cardinality_can_be_disabled_through_readconfig(
+      self, assert_cardinality):
+    train_info = self._write_tfrecord('train', 5, 'abcdefghijkl')
+    self.reader.read(
+        name='mnist',
+        instructions='train',
+        split_infos=[train_info],
+        read_config=read_config_lib.ReadConfig(assert_cardinality=False))
+    assert not assert_cardinality.called
+
+
+def test_shard_api():
+  si = tfds.core.SplitInfo(
+      name='train',
+      shard_lengths=[10, 20, 13],
+      num_bytes=0,
+  )
+  fi = [
+      shard_utils.FileInstruction(
+          filename='ds_name-train.tfrecord-00000-of-00003',
+          skip=0,
+          take=-1,
+          num_examples=10,
+      ),
+      shard_utils.FileInstruction(
+          filename='ds_name-train.tfrecord-00001-of-00003',
+          skip=0,
+          take=-1,
+          num_examples=20,
+      ),
+      shard_utils.FileInstruction(
+          filename='ds_name-train.tfrecord-00002-of-00003',
+          skip=0,
+          take=-1,
+          num_examples=13,
+      ),
+  ]
+  sd = splits.SplitDict([si], dataset_name='ds_name')
+  assert sd['train[0shard]'].file_instructions == [fi[0]]
+  assert sd['train[1shard]'].file_instructions == [fi[1]]
+  assert sd['train[-1shard]'].file_instructions == [fi[-1]]
+  assert sd['train[-2shard]'].file_instructions == [fi[-2]]
+  assert sd['train[:2shard]'].file_instructions == fi[:2]
+  assert sd['train[1shard:]'].file_instructions == fi[1:]
+  assert sd['train[-1shard:]'].file_instructions == fi[-1:]
+  assert sd['train[1:-1shard]'].file_instructions == fi[1:-1]
+
 
 if __name__ == '__main__':
   testing.test_main()
