@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,9 @@
 from tensorflow_datasets.structured.amazon_us_reviews import AmazonUSReviews
 from tensorflow_datasets.structured.cherry_blossoms import CherryBlossoms
 from tensorflow_datasets.structured.covid19 import Covid19
+from tensorflow_datasets.structured.cs_restaurants import CSRestaurants
 from tensorflow_datasets.structured.dart import Dart
+from tensorflow_datasets.structured.diamonds import Diamonds
 from tensorflow_datasets.structured.e2e_cleaned import E2eCleaned
 from tensorflow_datasets.structured.efron_morris_75 import EfronMorris75
 from tensorflow_datasets.structured.forest_fires import ForestFires
@@ -29,6 +31,7 @@ from tensorflow_datasets.structured.howell import Howell
 from tensorflow_datasets.structured.iris import Iris
 from tensorflow_datasets.structured.kddcup99 import Kddcup99
 from tensorflow_datasets.structured.movielens import Movielens
+from tensorflow_datasets.structured.penguins import Penguins
 from tensorflow_datasets.structured.proteinnet import ProteinNet
 from tensorflow_datasets.structured.radon import Radon
 from tensorflow_datasets.structured.rock_you import RockYou
