@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,19 +16,18 @@
 """Test case util to test `tfds.features.FeatureConnector`."""
 
 import contextlib
+import dataclasses
 import functools
 from typing import Any, Optional, Type
 
-import dataclasses
 import dill
 import numpy as np
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import dataset_utils
-from tensorflow_datasets.core import example_parser
-from tensorflow_datasets.core import example_serializer
 from tensorflow_datasets.core import features
 from tensorflow_datasets.core import utils
+from tensorflow_datasets.core.features import feature as feature_lib
 from tensorflow_datasets.testing import test_case
 from tensorflow_datasets.testing import test_utils
 
@@ -199,6 +198,21 @@ class FeatureExpectationsTestCase(SubTestCase):
             skip_feature_tests=skip_feature_tests,
             test_attributes=test_attributes,
         )
+      with self._subTest('feature_proto_roundtrip'):
+        with test_utils.tmp_dir() as config_dir:
+          feature_proto = feature.to_proto()
+          feature.save_metadata(config_dir, feature_name=None)
+          new_feature = feature_lib.FeatureConnector.from_proto(feature_proto)
+          new_feature.load_metadata(config_dir, feature_name=None)
+          self._assert_feature(
+              feature=new_feature,
+              shape=shape,
+              dtype=dtype,
+              tests=tests,
+              serialized_info=serialized_info,
+              skip_feature_tests=skip_feature_tests,
+              test_attributes=test_attributes,
+          )
 
   def _assert_feature(
       self,
@@ -345,23 +359,14 @@ class FeatureExpectationsTestCase(SubTestCase):
 
 def features_encode_decode(features_dict, example, decoders):
   """Runs the full pipeline: encode > write > tmp files > read > decode."""
-  # Encode example
-  encoded_example = features_dict.encode_example(example)
-
   # Serialize/deserialize the example
-  specs = features_dict.get_serialized_info()
-  serializer = example_serializer.ExampleSerializer(specs)
-  parser = example_parser.ExampleParser(specs)
+  serialized_example = features_dict.serialize_example(example)
 
-  serialized_example = serializer.serialize_example(encoded_example)
-  ds = tf.data.Dataset.from_tensors(serialized_example)
-  ds = ds.map(parser.parse_example)
-
-  # Decode the example
   decode_fn = functools.partial(
-      features_dict.decode_example,
+      features_dict.deserialize_example,
       decoders=decoders,
   )
+  ds = tf.data.Dataset.from_tensors(serialized_example)
   ds = ds.map(decode_fn)
 
   if tf.executing_eagerly():
