@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2021 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ from unittest import mock
 
 from absl import logging
 import numpy as np
-import tensorflow.compat.v2 as tf
+import tensorflow as tf
 
 from tensorflow_datasets.core import dataset_builder
 from tensorflow_datasets.core import decode
@@ -56,6 +56,8 @@ class MockPolicy(enum.Enum):
 @contextlib.contextmanager
 def mock_data(
     num_examples: int = 1,
+    num_sub_examples: int = 1,
+    max_value: Optional[int] = None,
     *,
     policy: MockPolicy = MockPolicy.AUTO,
     as_dataset_fn: Optional[Callable[..., tf.data.Dataset]] = None,
@@ -123,10 +125,14 @@ def mock_data(
   this functions, please open a new issue on our Github.
 
   Args:
-    num_examples: number of fake example to generate.
+    num_examples: Number of fake example to generate.
+    num_sub_examples: Number of examples to generate in nested Dataset features.
+    max_value: The maximum value present in generated tensors; if max_value is
+      None or it is set to 0, then random numbers are generated from the range
+      from 0 to 255.
     policy: Strategy to use to generate the fake examples. See
       `tfds.testing.MockPolicy`.
-    as_dataset_fn: if provided, will replace the default random example
+    as_dataset_fn: If provided, will replace the default random example
       generator. This function mock the `FileAdapterBuilder._as_dataset`
     data_dir: Folder containing the metadata file (searched in
       `data_dir/dataset_name/version`). Overwrite `data_dir` kwargs from
@@ -179,6 +185,7 @@ def mock_data(
 
     # Partial decoding
     if isinstance(decoders, decode.PartialDecoding):
+      # TODO(epot): Should be moved inside `features.decode_example`
       features = decoders.extract_features(self.info.features)
       decoders = decoders.decoders
     # Full decoding (all features decoded)
@@ -186,31 +193,40 @@ def mock_data(
       features = self.info.features
       decoders = decoders  # pylint: disable=self-assigning-variable
 
-    if decoders is None:
-      generator_cls = RandomFakeGenerator
-      specs = features.get_tensor_info()
-      decode_fn = lambda ex: ex  # identity
-    else:
+    has_nested_dataset = any(
+        isinstance(f, features_lib.Dataset)
+        for f in features._flatten(features))  # pylint: disable=protected-access
+    if decoders is not None or has_nested_dataset:
       # If a decoder is passed, encode/decode the examples.
       generator_cls = EncodedRandomFakeGenerator
       specs = features.get_serialized_info()
       decode_fn = functools.partial(features.decode_example, decoders=decoders)
+    else:
+      generator_cls = RandomFakeGenerator
+      specs = features.get_tensor_info()
+      decode_fn = lambda ex: ex  # identity
 
     ds = tf.data.Dataset.from_generator(
         # `from_generator` takes a callable with signature () -> iterable
         # Recreating a new generator each time ensure that all pipelines are
         # using the same examples
-        lambda: generator_cls(features=features, num_examples=num_examples),
+        # pylint: disable=g-long-lambda]
+        lambda: generator_cls(
+            features=features,
+            num_examples=num_examples,
+            num_sub_examples=num_sub_examples,
+            max_value=max_value),
+        # pylint: enable=g-long-lambda]
         output_types=tf.nest.map_structure(lambda t: t.dtype, specs),
         output_shapes=tf.nest.map_structure(lambda t: t.shape, specs),
     )
     ds = ds.apply(tf.data.experimental.assert_cardinality(num_examples))
-    ds.map(decode_fn, num_parallel_calls=tf.data.experimental.AUTOTUNE)
+    ds = ds.map(decode_fn, num_parallel_calls=tf.data.experimental.AUTOTUNE)
 
     if read_config and read_config.add_tfds_id:
       ds_id = tfrecords_reader._make_id_dataset(  # pylint: disable=protected-access
           filename=f'{self.name}-split.tfrecord-00000-of-00001',
-          start_index=0,
+          start_index=0,  # pytype: disable=wrong-arg-types
       )
       ds = tf.data.Dataset.zip((ds, ds_id))
       ds = ds.map(lambda ex, id: {'tfds_id': id, **ex})
@@ -277,11 +293,18 @@ def mock_data(
 class RandomFakeGenerator(object):
   """Generator of fake examples randomly and deterministically generated."""
 
-  def __init__(self, features, num_examples, seed=0):
+  def __init__(self,
+               features,
+               num_examples: int,
+               num_sub_examples: int = 1,
+               max_value: Optional[int] = None,
+               seed: int = 0):
     self._rgn = np.random.RandomState(seed)  # Could use the split name as seed
     self._py_rng = random.Random(seed)
     self._features = features
     self._num_examples = num_examples
+    self._num_sub_examples = num_sub_examples
+    self._max_value = max_value
 
   def _generate_random_string_array(self, shape):
     """Generates an array of random strings."""
@@ -297,12 +320,24 @@ class RandomFakeGenerator(object):
     return np.array([rand_str() for _ in range(np.prod(shape, dtype=np.int32))
                     ]).reshape(shape)
 
-  def _generate_random_array(self, feature, tensor_info):
+  def _generate_random_obj(self, feature, tensor_info):
     """Generates a random tensor for a single feature."""
     # TODO(tfds): Could improve the fake generatiion:
     # * Use the feature statistics (min, max)
     # * For Sequence features
     # * For Text
+
+    # First we deal with the case of sub-datasets:
+    if isinstance(feature, features_lib.Dataset):
+      # For sub-datasets self._num_sub_examples examples are generated.
+      generator = RandomFakeGenerator(
+          feature.feature,
+          num_examples=self._num_sub_examples,
+          num_sub_examples=1,
+          max_value=self._max_value)
+      # Returns the list of examples in the nested dataset.
+      return list(generator)
+
     shape = [  # Fill dynamic shape with random values
         self._rgn.randint(5, 50) if s is None else s for s in tensor_info.shape
     ]
@@ -310,6 +345,8 @@ class RandomFakeGenerator(object):
       max_value = feature.num_classes
     elif isinstance(feature, features_lib.Text) and feature.vocab_size:
       max_value = feature.vocab_size
+    elif self._max_value:
+      max_value = self._max_value
     else:
       max_value = 255
 
@@ -331,11 +368,11 @@ class RandomFakeGenerator(object):
     root_feature = self._features
     flat_features = root_feature._flatten(root_feature)  # pylint: disable=protected-access
     flat_tensor_info = root_feature._flatten(root_feature.get_tensor_info())  # pylint: disable=protected-access
-    flat_np = [
-        self._generate_random_array(feature, tensor_info)
+    flat_objs = [
+        self._generate_random_obj(feature, tensor_info)
         for feature, tensor_info in zip(flat_features, flat_tensor_info)
     ]
-    return root_feature._nest(flat_np)  # pylint: disable=protected-access
+    return root_feature._nest(flat_objs)  # pylint: disable=protected-access
 
   def __iter__(self):
     """Yields all fake examples."""
