@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2022 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,13 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lint as: python3
 """Video datasets."""
 
 from tensorflow_datasets.video.bair_robot_pushing import BairRobotPushingSmall
+from tensorflow_datasets.video.davis import Davis
 from tensorflow_datasets.video.moving_mnist import MovingMnist
 from tensorflow_datasets.video.robonet import Robonet
 from tensorflow_datasets.video.starcraft import StarcraftVideo
 from tensorflow_datasets.video.starcraft import StarcraftVideoConfig
+from tensorflow_datasets.video.tao import Tao
 from tensorflow_datasets.video.ucf101 import Ucf101
 from tensorflow_datasets.video.ucf101 import Ucf101Config
+from tensorflow_datasets.video.youtube_vis import YoutubeVis
